
{% extends "base.html" %}
{% block content %}
  <div class="container">

  <h1> <a href="/"> Donkey </a> </h1>

    <p> Your donkey is ready to ride</p>

    <p> Using this web interface you can do the following </p>

    <ul> 
      <li> Drive your car. </li>
      <li> View data collected from your car. </li>
    </ul>


    <p> Features we are working on include: </p>
    <ul> 
      <li> Train & debug models from web. </li>
      <li> Browse library of driving models and download them. </li>
      <li> Combine models </li>

    </ul>
  </div>
{% end %}
